@@ -10,7 +10,7 @@
 #ABSTRACT: Common methods for App::Cme
 
 package App::Cme::Common;
-$App::Cme::Common::VERSION = '1.036';
+$App::Cme::Common::VERSION = '1.039';
 use strict;
 use warnings;
 use 5.10.1;
@@ -25,9 +25,6 @@ use Encode qw(decode_utf8);
 
 my @store;
 
-## no critic (Variables::ProhibitPackageVars)
-$::_use_log4perl_to_warn = 1;
-
 sub cme_global_options {
   my ( $class, $app ) = @_;
 
@@ -217,8 +214,14 @@ sub run_tk_ui {
     $instance->on_message_cb(sub{$cmu->show_message(@_);});
 
     if ($opt->{open_item}) {
-        my $obj = $instance->grab($opt->{open_item});
-        $cmu->force_element_display($obj);
+        my $obj = $instance->grab(step => $opt->{open_item}, autoadd => 0);
+        # using afterIdle avoids geometry problem where the right side
+        # of the widget is not visible
+        $mw->afterIdle( sub {
+            $cmu->force_element_display($obj);
+            my $path = $cmu->{tktree}->selectionGet;
+            $cmu->create_element_widget('edit', $path);
+        })
     }
 
     &MainLoop;    # Tk's
@@ -272,7 +275,7 @@ App::Cme::Common - Common methods for App::Cme
 
 =head1 VERSION
 
-version 1.036
+version 1.039
 
 =head1 SYNOPSIS
 
